//+------------------------------------------------------------------+
//|                                     Macd with EMA BDv8_12_31.mq4 |
//|                                                             Jeff |
//|                                                  wolfboy@att.net |
//+------------------------------------------------------------------+
#property copyright "Jeff"
#property link      "wolfboy@att.net"

// Macd with EMA BDv8_12_31:  Original indicator modified for the Black Dog Trading System.

// Charts the MACD along with the EMA of the MACD Signal.

//---- indicator settings
#property  indicator_separate_window
#property  indicator_buffers   3
#property  indicator_color1    Silver
#property  indicator_color2    Red
#property  indicator_color3    Yellow
#property  indicator_style3    2 
#property  indicator_width1    1
#property indicator_levelcolor DarkSlateGray

//
//
//
//
//


extern string TimeFrame     = "Current time frame";
extern int    FastEMA       = 10;
extern int    SlowEMA       = 20;
extern int    SignalEMA     = 1;
extern int    MAofSignalPer = 7;
extern bool   Interpolate   = true;

//
//
//
//
//

double macd[];
double signal[];
double MAofsignal[];

//
//
//
//
//

string IndicatorFileName;
bool   returnBars;
bool   calculateValue;
bool   calculateMa;
int    timeFrame;
string shortName;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   SetIndexBuffer(0,macd); SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1,signal);
   SetIndexBuffer(2,MAofsignal);
   SetLevelValue(0,0);
   
   SetIndexLabel(0,"MACD");
   SetIndexLabel(1,"Signal");
   SetIndexLabel(2,"MAofSignal");
   
      FastEMA       = MathMax(FastEMA,1);
      SlowEMA       = MathMax(SlowEMA,1);
      SignalEMA     = MathMax(SignalEMA,1);
      MAofSignalPer = MathMax(MAofSignalPer,1);
   //
   //
   //
   //
   //
   
      IndicatorFileName = WindowExpertName();
      returnBars        = (TimeFrame == "returnBars");     if (returnBars)     return(0);
      calculateValue    = (TimeFrame == "calculateValue"); if (calculateValue) return(0);
      timeFrame         = stringToTimeFrame(TimeFrame);   

   //
   //
   //
   //
   //
   
   IndicatorShortName(timeFrameToString(timeFrame)+"  MACD("+FastEMA+", "+SlowEMA+", "+SignalEMA+", "+MAofSignalPer+")");
   return(0);
}
      
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int i,limit;
   int counted_bars = IndicatorCounted();

   if(counted_bars < 0) return(-1);
   if(counted_bars > 0) counted_bars--;
             limit=MathMin(Bars-counted_bars,Bars-1);
             if (returnBars) { macd[0] = limit; return(0); }

   //
   //
   //
   //
   //

   if (calculateValue || timeFrame == Period())
   {
      for(i=limit; i>=0; i--) macd[i]       = iMA(NULL,0,FastEMA,0,MODE_EMA,PRICE_CLOSE,i)- iMA(NULL,0,SlowEMA,0,MODE_EMA,PRICE_CLOSE,i);
      for(i=limit; i>=0; i--) signal[i]     = iMAOnArray(macd,Bars,SignalEMA,0,MODE_EMA,i);
      for(i=limit; i>=0; i--) MAofsignal[i] = iMAOnArray(signal,Bars,MAofSignalPer,0,MODE_EMA,i);

   return(0);
   }
   
   //
   //
   //
   //
   //
   
   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,IndicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for(i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         macd[i]       = iCustom(NULL,timeFrame,IndicatorFileName,"calculateValue",FastEMA,SlowEMA,SignalEMA,MAofSignalPer,0,y);
         signal[i]     = iCustom(NULL,timeFrame,IndicatorFileName,"calculateValue",FastEMA,SlowEMA,SignalEMA,MAofSignalPer,1,y);
         MAofsignal[i] = iCustom(NULL,timeFrame,IndicatorFileName,"calculateValue",FastEMA,SlowEMA,SignalEMA,MAofSignalPer,2,y);
            
         //
         //
         //
         //
         //
      
         if (!Interpolate || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;

         //
         //
         //
         //
         //

         datetime time = iTime(NULL,timeFrame,y);
            for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
            for(int k = 1; k < n; k++)
            {
               macd[i+k]       = macd[i]       +(macd[i+n]       - macd[i])       * k/n;
               signal[i+k]     = signal[i]     +(signal[i+n]     - signal[i])     * k/n;
               MAofsignal[i+k] = MAofsignal[i] +(MAofsignal[i+n] - MAofsignal[i]) * k/n;
            }               
   }               

   //
   //
   //
   //
   //
            
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string StringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}

